------------------------------------------------
------------------------------------------------
-- DOCUMENTATION

helptext=[[
<html>
<head>
 <title>Function Plot</title>
 </head>
<body bgcolor=#ffffff text=#000000 link=#880000 vlink=#884444 alink=#8844442> 

<h1><a name="Contents"> Contents </a></h1>

<h3><a href="#General"> General Description </a> </h3>
<h3><a href="#Grids"> Defining Grids </a> </h3>
<h3><a href="#Functions"> Defining Functions </a></h3>
<h3><a href="#GUI"> User Controls </a></h3>
<h3><a href="#About"> About </a> </h3>
<hr>
<h3><a name="General"> General Description </a> </h3>

<p align=justify>

The function plot  program allows to plot functions  on two- and
three-dimensional  simplicial  grids. 

<h3><a name="Grids"> Defining Grids </a> </h3>
<h3><a name="Functions"> Defining Functions </a></h3>
<p>
A function is defined using a function definition in the Lua language.
Example:<br>
<code>
function f(x,y) return sin(3*x)*cos(3*y) end
</code> <br>
This function definition can edited in the edit window. 
It can  be stored in a file with the extension <tt>.func</tt>. 
<tt>.func</tt> files can be edited with any ASCII editor.
The main convention is that the name of the function has to be <tt>f</tt>.
<p>


<h3><a name="GUI"> User controls </a></h3>
<h3><a name="About"> About </a> </h3>
Function  plot uses  pdelib, a  toolbox  for the  solution of  partial
differential equations.  It makes use  of the Lua  scripting language,
and of the FLTK GUI toolkit


</body>
</html>
]]

--------------------------------------------------------------------
--------------------------------------------------------------------
-- COMMAND LINE, START UP
import "math"



processargs(arg)

if arg.files[1] then 
   fname=arg.files[1]
   inp=io.open(arg.files[1])
   func=inp:read("*a")
   inp:close()
end

-- Default grid
if arg.values.grid then 
	    sgrid=[[grid=gRead("]]..arg.values.grid..[[")]]
else
   sgrid=[[xgrid=Grid{
      points={{-1,-1},  {1,-1}, {1,1},{-1,1}},
      facets={ {1,2},{2,3},{3,4},{4,1}},
      facetregions={1,2,3,4},
      regionpoints={{0.5,0.5}},
      regionnumbers={1},
      regionvolumes={0.001},
      flags="paAq30",
   }]]
end

-- Default function
fdecl= "function f(x,y)"
fstub="sin(3*x)*cos(3*y)"
if arg.values.f then fstub=arg.values.f end

function regrid()
   local input=fl_get_input(inputs.grid)
   A=fl_get_slider_value(objects.A)
   B=fl_get_slider_value(objects.B)
   C=fl_get_slider_value(objects.C)
   D=fl_get_slider_value(objects.D)

   local defgrid=loadstring(input)
   
   if defgrid then 
      defgrid()
   else 
      fl_show_alert("Syntax Error in Grid Text.","Please correct.","")
   end


   dim=gSpaceDim(xgrid)
   pt=gCoord(xgrid)
   space=gsCreateFV(xgrid,gNode)
   v=da1CreateS(space)
   n=da2NRow(pt)
end


function calc()
   local input=fl_get_input(inputs.func)
   A=fl_get_slider_value(objects.A)
   B=fl_get_slider_value(objects.B)
   C=fl_get_slider_value(objects.C)
   D=fl_get_slider_value(objects.D)
   
   
   local defun=loadstring(input)
   
   if defun then 
      defun()
   else 
      fl_show_alert("Syntax Error in Function Text.","Please correct.","")
   end
   
   if dim==2 then
      for i=1,n do	
	 v[i]=f(pt[i][1],pt[i][2],0)
      end
   else
      for i=1,n do	
	 v[i]=f(pt[i][1],pt[i][2],pt[i][3])
      end
   end
   return v
end



immediate=false

function icalc()
   if immediate then
      if visible.func then calc() end
      if visible.grid then regrid() calc() end
   end
end


---------------------------------------------------
--------------------------------------------------
-- GUI

forms={}
inputs={}
objects={}
callbacks={}
groups={}
visible={}

function forms.create_main()
   forms.main=fl_bgn_form(FL_UP_BOX,980,735)
   fl_set_form_title(forms.main,"Function plot")
--   fl_set_form_position(forms.main,1,1)
   viewer=glViewer{w=800,h=600,x=0+2,y=(735-600)-2}
   
   
   
   --Help=fl_add_button(FL_NORMAL_BUTTON,700,700,100,25,"Help");
   --Exit=fl_add_button(FL_NORMAL_BUTTON,700,650,100,25,"Exit");
   
   
   local s0=200
   local sh=500
   objects.A=fl_add_valslider(FL_VERT_SLIDER,(805+0*30),s0,30,sh,"A");
   fl_set_object_lalign(objects.A,FL_ALIGN_TOP)
   fl_set_slider_bounds(objects.A,0,1);
   fl_set_slider_value(objects.A,0.5);
   callbacks.A=icalc
   
   objects.B=fl_add_valslider(FL_VERT_SLIDER,(805+1*30),s0,30,sh,"B");
   fl_set_object_lalign(objects.B,FL_ALIGN_TOP)
   fl_set_slider_bounds(objects.B,0,1);
   fl_set_slider_value(objects.B,0.5);
   callbacks.B=icalc
   
   objects.C=fl_add_valslider(FL_VERT_SLIDER,(805+2*30),s0,30,sh,"C");
   fl_set_object_lalign(objects.C,FL_ALIGN_TOP)
   fl_set_slider_bounds(objects.C,0,1);
   fl_set_slider_value(objects.C,0.5);
   callbacks.C=icalc

   objects.D=fl_add_valslider(FL_VERT_SLIDER,(805+3*30),s0,30,sh,"D");
   fl_set_object_lalign(objects.D,FL_ALIGN_TOP)
   fl_set_slider_bounds(objects.D,0,1);
   fl_set_slider_value(objects.D,0.5);
   callbacks.D=icalc

   objects.immediate=fl_add_button(FL_TOGGLE_BUTTON,805,s0-35,120,25,"Immediate");
   callbacks.immediate=function() 
			  if immediate then immediate=false else immediate=true end end

   
   
   tabs=fl_bgn_tabs(55,2,945,125,"")
   
   groups.func=fl_bgn_group()
   fl_set_group_label(groups.func,"Func")
   fl_add_box(FL_NO_BOX,55,97,745,5,"");
   inputs.func=fl_add_input(FL_MULTILINE_INPUT,55,2,745,95,"");
   if not func then fl_set_input(inputs.func,fdecl.." return "..fstub.." end") 
   else
      fl_set_input(inputs.func,func)
   end

   objects.func_load=fl_add_button(FL_NORMAL_BUTTON,805,050+2,100,25,"Load Func");
   callbacks.func_load=function()
      local fname=
	 fl_show_file_selector("Select Function",".","Function (*.func)","") 
      if fname then
	 inp=io.open(fname)
	 fl_set_input(inputs.func,inp:read("*a"))
	 F=calc()
	 inp:close()
      end
   end
   
   objects.func_save=fl_add_button(FL_NORMAL_BUTTON,805,025+2,100,25,"Save Func");
   callbacks.func_save=function()
      local fname=fl_show_file_selector("Save Function",".","Function (*.func)",nil) 
      if fname then
	 outp=io.open(fname,"w")
	 outp:write(fl_get_input(inputs.func))
	 outp:close()
      end
   end

   objects.func_calc=fl_add_button(FL_NORMAL_BUTTON,805,000+2,100,25,"Calc Func");
   callbacks.func_calc=calc
   fl_end_group()
   
   groups.grid=fl_bgn_group()
   fl_set_group_label(groups.grid,"Grid")
   inputs.grid=fl_add_input(FL_MULTILINE_INPUT,55,2,745,95,"");
   fl_set_input(inputs.grid,sgrid)

   objects.grid_load=fl_add_button(FL_NORMAL_BUTTON,805,050,100,25,"Load Grid");
   callbacks.grid_load=function()
      local gname=
	 fl_show_file_selector(
			       "Select Grid",
			       ".",
			       "Grids (*.{sg,grid,xgrid,ele})",nil) 
      if gname then
	 local stub,ext=splitfilename(gname)
	 if (ext=="grid") then
	    inp=io.open(gname)
	    fl_set_input(inputs.grid,inp:read("*a"))
	    inp:close()
         else
	    fl_set_input(inputs.grid,[[grid=gRead("]]..gname..[[")]])
         end
	 regrid()
	 F=calc()
      end
   end

   objects.grid_save=fl_add_button(FL_NORMAL_BUTTON,805,025+2,100,25,"Save Grid");
   callbacks.grid_save=function()
      local gname=fl_show_file_selector("Save Grid",".","Grid (*.grid)",nil) 
      if gname then
	 outp=io.open(gname,"w")
	 outp:write(fl_get_input(inputs.grid))
	 outp:close()
      end
   end
 
   objects.grid_calc=fl_add_button(FL_NORMAL_BUTTON,805,000+2,100,25,"Calc Grid");
   callbacks.grid_calc=function() regrid() F=calc() end
   fl_end_group()

   fl_end_group()
   fl_end_tabs()

   fl_tabs_set_visible(tabs,groups.func)
   
   fl_end_form()
end


function forms.create_help()
   forms.help=fl_bgn_form(FL_UP_BOX,400,300)
   fl_set_form_title(forms.help,"Help")
   helpview=fl_add_helpview(0,0,400,300,"")
   fl_set_helpview_value(helpview,helptext)
   fl_end_form()
end

forms.create_main()
fl_show_form(forms.main,FL_PLACE_FREE,1,"pdelib2 Function plot");


regrid()
F=calc()

---------------------------------------------------------------
-- MAIN LOOP

while true do
   
   if title then glvSetTitle(viewer,title) end
   
   visible.func=false
   visible.grid=false
   if swig_type(tabs)=="Fl_Widget"  then
      if swig_equals(fl_tabs_visibleW(tabs),groups.func) then
	 da1View(viewer,F)
	 visible.func=true
      else
	 gView(viewer,xgrid)
	 visible.grid=true
      end
   else
      if swig_equals(fl_tabs_visible(tabs),groups.func) then
	 da1View(viewer,F)
	 visible.func=true
      else
	 gView(viewer,xgrid)
	 visible.grid=true
      end
   end
   
   glvShow()
   obj = fl_do_forms();
   for i,v in pairs(objects) do
      if swig_equals(obj,v) then callbacks[i]()  end
   end
end	
  
  
  
  
  




