processargs(arg)
job=arg.values.job
grid=arg.values.grid
import "math"

function help()
print[[

SYNOPSIS
        semiview:  3D Semiconductor device simulator  output viewer

USAGE  
      ldf|pdelib2  semiview [-h] job=... grid=...  bias=... spec=... 

SWITCHES

      -h  Show this help

COMMAND LINE VALUES
   
      job   Name prefix of index file (.gli) for device data.
      
      grid  Name of grid file (.grid, .sg). Default is taken from job index.
      
      bias  Value of bias to show. If not given, an index is printed

      spec  Name of species to be shown. If not given, an index is printed.
            Alternatively, spec can contain a Lua expression to be evaluated
            within a loop over all grid nodes, e.g. "log10(pot[i])".


EXAMPLES

    * ldf semiview job=/Home/3dbe/gaertner/rrD/rrD3-54zebra/rrD3-54zebra156654back-30          
      just prints the index of job  rrD3-54zebra156654back-30:
      
      grid:   /Home/3dbe/gaertner/rrD/rrD3-54zebra/rrD3-54zebra.1.sg
      bias:   0.001    spec:       ele-st.P         pos:   0
      bias:   0.001    spec:       dens_N   pos:   1253232
      bias:   0.001    spec:       dens_P   pos:   2506464

    * ldf semiview bias=0.001 spec=dens_N job=/Home/3dbe/gaertner/rrD/rrD3-54zebra/rrD3-54zebra156654back-30
      visualized data of species dens_N.

    * ldf semiview bias=0.001 spec="dens_N[i]-dens_P[i]" job=/Home/3dbe/gaertner/rrD/rrD3-54zebra/rrD3-54zebra156654back-30
      visualizes the difference betwenn dens_N and dens_P.


BUGS 
    Still Quite Some.

SEE ALSO
   pdelib2: http://www.wias-berlin.de/software/pdelib
   Lua:     http://www.lua.org
   
]]

mpeg_param=[[
PATTERN  IBBPBBPBBPBBPBB
GOP_SIZE  30
SLICES_PER_FRAME 1
PIXEL	HALF
RANGE	10
PSEARCH_ALG	LOGARITHMIC
BSEARCH_ALG	CROSS2
IQSCALE 8
PQSCALE 10
BQSCALE  25
REFERENCE_FRAME	ORIGINAL
]]

os.exit(1)
end

if arg.switches.h then help() end


if not job then help() end


---------------------------------------------------
-- scan index for positions etc.
idx=io.open(job..".gli")
ign={
   "DataSeriesIndex",
   "#",
   "ITEMSIZE",
   "STEPS",
   "END"
}


function ignore(l,ign)
   for i=1,table.getn(ign) do
      if string.find(l,ign[i]) then return 1 end
   end
   return nil
end

function split (line,regexp)
   local tbn={}   
   string.gsub(line,regexp,function(s) table.insert(tbn,s) end)
   return tbn
end





xspecies={} -- nspec x nbias 2D array of records for species per bias



for l in idx:lines() do
   if not ignore(l,ign) then
      l=split(l,"([^%s]+)")
      if l[1]=="GRID" then 
	 if not grid then grid=l[2] end
      else
	 bias=tonumber(l[1])
	 species=l[4]
	 if not xspecies[species] then
	    xspecies[species]={} ibias=1 
	    _G[species]={}
	    _G[species].biases=xspecies[species]
	 else
	    ibias=table.getn(xspecies[species])+1
	 end
	 xspecies[species][ibias]={spec=species, ibias=ibias, bias=bias, len=tonumber(l[2]), pos=tonumber(l[3])}   
      end
   end
end
idx:close()





if not grid then help() end

if arg.values.spec then
   spec=arg.values.spec
end


grid=gRead(grid)
space=gsCreateFV(grid,gNode)
gPrintStatus(grid)


n=gNElements(grid,gNode)
f=da1CreateS(space)



function resetbrowser(species)
   fl_set_input(inputs.func,"load("..species..",ibias)\n"..
		"for i=1,n do f[i]="..species.."[ibias][i] end\n\n"..
		   [[title="]]..species..[[, bias="..]].."bias"
	     ) 

   fl_clear_browser(objects.bias_browser)
   iline=0
   
   for i,v in pairs(xspecies[species]) do
	 iline=iline+1
	 fl_add_browser_line(objects.bias_browser, string.format("[%03d]: %10.5g",
								 xspecies[species][i].ibias,
								 xspecies[species][i].bias))
   end
   nbias=iline
end

function load(xspec,xibias)
   if not xspec[xibias] and xspec.biases[xibias] then
      xspec[xibias]=da1CreateS(space)
      data=Data{xspec[xibias],
	 encoding="xdr",
	 file=job..".gld",
	 pos=xspec.biases[xibias].pos,
	 nrow=n
      }
      if not data then 
	 error("unable to open file"..file)
	 os.exit()
      end
   end
   if xspec.biases[xibias] then
      bias=xspec.biases[xibias].bias
   end
   if not xspec[xibias] and xspec.biases[xibias] then
      return xspec[xibias]
   end
   return nil
end


function unload(xspec,xibias)
   xspec[xibias]=nil
   collectgarbage()
end

function calc()
   title="bias="..bias
   local defun=loadstring(fl_get_input(inputs.func))
   if defun then 
      defun()
   else 
      fl_show_alert("Syntax Error in Function Text.","Please correct.","")
   end
   if title then glvSetTitle(viewer,title) end
end





forms={}
inputs={}
objects={}
callbacks={}
groups={}
visible={}



forms.main=fl_bgn_form(FL_UP_BOX,980,735)
   fl_set_form_title(forms.main,"Function plot")
   viewer=glViewer{w=750,h=600,x=0+2,y=(735-600)-2}
   local s0=(735-600)-2
   local sh=600-25

   
   inputs.func=fl_add_input(FL_MULTILINE_INPUT,55,2,400,100,"Edit expression");
   fl_set_object_lalign(inputs.func,FL_ALIGN_LEFT_TOP) ;
   
   objects.func_load=fl_add_button(FL_NORMAL_BUTTON,455,075+2,100,25,"Load Func");
   objects.func_save=fl_add_button(FL_NORMAL_BUTTON,455,050+2,100,25,"Save Func");
   objects.func_calc=fl_add_button(FL_NORMAL_BUTTON,455,025+2,100,25,"Calc Func");

   callbacks.func_load=function()
      local fname=
	 fl_show_file_selector("Select Function",".","Function (*.func)","") 
      if fname then
	 inp=io.open(fname)
	 fl_set_input(inputs.func,inp:read("*a"))
	 F=calc()
	 inp:close()
      end
   end
   
   callbacks.func_save=function()
      local fname=fl_show_file_selector("Save Function",".","Function (*.func)",nil) 
      if fname then
	 outp=io.open(fname,"w")
	 outp:write(fl_get_input(inputs.func))
	 outp:close()
      end
   end
   callbacks.func_calc=calc
   

   objects.animate=fl_add_button(FL_TOGGLE_BUTTON,   555,075+2,100,25,"Animate");
   objects.ppm=fl_add_button(FL_TOGGLE_BUTTON,       555,050+2,100,25,"Record PPM");
   objects.mpeg=fl_add_button(FL_TOGGLE_BUTTON,      555,025+2,100,25,"Record MPEG");
   objects.keepminmax=fl_add_button(FL_TOGGLE_BUTTON,555,000+2,100,25,"Keep Min/Max");
   
   
   objects.bias_browser=fl_add_browser(FL_SELECT_BROWSER,755,s0,200,sh,"Bias List")
   fl_set_object_lalign(objects.bias_browser,FL_ALIGN_LEFT_TOP) ;
   callbacks.bias_browser=function()
			     ibias=fl_get_browser(objects.bias_browser)
			     fl_select_browser_line(objects.bias_browser,ibias)
			     calc()
			  end		     
   
   objects.species_browser=fl_add_browser(FL_SELECT_BROWSER,755,2,200,s0-20,"Species List")
   fl_set_object_lalign(objects.species_browser,FL_ALIGN_LEFT_TOP) ;
   
   callbacks.species_browser=
      function()
	 local xline=fl_get_browser(objects.species_browser)
	 species=fl_get_browser_line(objects.species_browser,xline)
	 fl_select_browser_line(objects.species_browser,xline)
	 resetbrowser(species)
      end 
fl_end_form()

fl_show_form(forms.main,FL_PLACE_FREE,1,"pdelib2 semiview");

iline=0
for xspec,tab in xspecies do
   iline=iline+1
   fl_add_browser_line(objects.species_browser,xspec)
   if species==xspec then fl_select_browser_line(objects.species_browser,iline) end
end

fl_select_browser_line(objects.bias_browser,1)






ibias=1
resetbrowser(species)
if arg.values.func then
   local fname=arg.values.func
   local inp=io.open(fname)
   fl_set_input(inputs.func,inp:read("*a"))
   calc()
   inp:close()
end
calc()


while true do
   
   if fl_get_button(objects.animate)==1  then
      
      if fl_get_button(objects.mpeg)==1  then
	 local fname=
	    fl_show_file_selector("Select MPEG File",".","MPEG (*.mpg)","") 
	 if fname then
	    glvStartMPEGRecording(viewer,fname,mpeg_params)
	 end
      end
      
      local oldibias=ibias
      for xibias=ibias,nbias do
	 if fl_get_button(objects.keepminmax)==1 then
	    aViewKeepMinMax(viewer)
	 end
	 fl_select_browser_line(objects.bias_browser,xibias)
	 ibias=xibias
	 calc()
	 da1View(viewer,f)
	 glvShow()
	 if fl_get_button(objects.mpeg)==1  then
	    glvDumpMPEG(viewer)
	 end
	 if fl_get_button(objects.ppm)==1  then
	    fname=string.format("%s-%05d.ppm",species,xibias)
	    glvDumpPPM(viewer,fname)
	    print("dump:", fname)
	 end
	 obj=fl_check_forms()
	 if swig_equals(obj,objects.animate) then break end
      end
      
      if fl_get_button(objects.mpeg)==1  then
	 glvStopMPEGRecording(viewer)
      end
      
      fl_set_button(objects.animate,0)
      ibias=oldibias
   else
      
      if fl_get_button(objects.keepminmax)==1 then
	 aViewKeepMinMax(viewer)
      end
      da1View(viewer,f)
      glvShow()
   end

   
   obj = fl_do_forms();
   

   for i,v in pairs(objects) do
      if swig_equals(obj,v)  and callbacks[i] then callbacks[i]()  end
   end

end





