help=[[

SYNOPSIS
        semiview2:  3D Semiconductor device simulator  output viewer

USAGE  
      ldf|pdelib2  semiview2 [-h] job=... grid=...  bias=... spec=... 

SWITCHES

      -h  Show this help

COMMAND LINE VALUES
   
      job   Name prefix of index file (.gli) for device data.
      
      grid  Name of grid file (.grid, .sg). Default is taken from job index.
      
      bias  Value of bias to show. If not given, an index is printed

      spec  Name of species to be shown. If not given, an index is printed.
            Alternatively, spec can contain a Lua expression to be evaluated
            within a loop over all grid nodes, e.g. "log10(pot[i])".


EXAMPLES

    * pdelib2 semiview2 job=/Home/3dbe/gaertner/rrD/rrD3-54zebra/rrD3-54zebra156654back-30          
      just prints the index of job  rrD3-54zebra156654back-30:
      
      grid:   /Home/3dbe/gaertner/rrD/rrD3-54zebra/rrD3-54zebra.1.sg
      bias:   0.001    spec:       ele-st.P         pos:   0
      bias:   0.001    spec:       dens_N   pos:   1253232
      bias:   0.001    spec:       dens_P   pos:   2506464

    * pdelib2 semiview2 bias=0.001 spec=dens_N job=/Home/3dbe/gaertner/rrD/rrD3-54zebra/rrD3-54zebra156654back-30
      visualized data of species dens_N.

    * pdelib2 semiview2 bias=0.001 spec="dens_N[i]-dens_P[i]" job=/Home/3dbe/gaertner/rrD/rrD3-54zebra/rrD3-54zebra156654back-30
      visualizes the difference betwenn dens_N and dens_P.


BUGS 
    Still Quite Some.

SEE ALSO
   pdelib2: http://www.wias-berlin.de/software/pdelib
   Lua:     http://www.lua.org
   
]]

processargs(arg)
job=arg.values.job
grid=arg.values.grid

if arg.switches.h then print(help) return end

if not job then print(help) return end

progSetErrorPublisherGUI()

import "math"

glv2SetStateFile(filePrefFile("semiview2.ini"))
glv2SetSemiviewState(true)

---------------------------------------------------
-- scan index for positions etc.
print("wrkorking-directory:",fileWrkDir())

job_gli=job..".gli"
if string.find(job,"%.gli$") then
 job_gli=job
 job=string.gsub(job_gli,"%.gli$","")
end
print("job:",job)
print("job_gli:",job_gli)





ign={
   "DataSeriesIndex",
   "#",
   "ITEMSIZE",
   "STEPS",
   "END"
}


function ignore(l,ign)
   for i=1,table.getn(ign) do
      if string.find(l,ign[i]) then return 1 end
   end
   return nil
end

function split (line,regexp)
   local tbn={}   
   string.gsub(line,regexp,function(s) table.insert(tbn,s) end)
   return tbn
end




function reload_gli()

 xspecies={} -- nspec x nbias 2D array of records for species per bias

 idx=io.open(job_gli)
 for l in idx:lines() do
   if not ignore(l,ign) then
      l=split(l,"([^%s]+)")
      if l[1]=="GRID" then 
	 if not grid then grid=l[2] end
      else
	 bias=tonumber(l[1])
	 species=l[4]
	 if not xspecies[species] then
	    xspecies[species]={} ibias=1 
	    _G[species]={}
	    _G[species].biases=xspecies[species]
	 else
	    ibias=table.getn(xspecies[species])+1
	 end
	 xspecies[species][ibias]={spec=species, ibias=ibias, bias=bias, len=tonumber(l[2]), pos=tonumber(l[3])}   
      end
   end
 end
 idx:close()

end

reload_gli()



if not grid then print(help) return end

if not fileIsAbsolute(grid) then
 -- relative path to working-dir
 grid=fileWrkDir().."/"..grid
end
print("grid:",grid)

if arg.values.spec then
   spec=arg.values.spec
end


grid=gRead(grid)
gPrintStatus(grid)


n=gNElements(grid,gNode)
f=da1CreateNode(grid)

function func_default(spec)
 return "load("..spec..",ibias)\n"..
 "for i=1,n do f[i]="..spec.."[ibias][i] end\n\n"..
   [[title="]]..spec..[[, bias="..]].."bias"
end

function bias_browser(species)
   iline=0
   local bb={}
   for i,v in pairs(xspecies[species]) do
	 iline=iline+1
	 bb[iline]=string.format("[%03d]: %10.5g",
				 xspecies[species][i].ibias,
				 xspecies[species][i].bias)
   end
   nbias=iline
   return bb
end

function load(xspec,xibias)
   if not xspec[xibias] and xspec.biases[xibias] then
      xspec[xibias]=da1CreateNode(grid)
      data=Data{xspec[xibias],
	 encoding="xdr",
	 file=job..".gld",
	 pos=xspec.biases[xibias].pos,
	 nrow=n
      }
      if not data then 
	 error("unable to open file"..file)
	 os.exit()
      end
   end
   if xspec.biases[xibias] then
      bias=xspec.biases[xibias].bias
   end
   if not xspec[xibias] and xspec.biases[xibias] then
      return xspec[xibias]
   end
   return nil
end


function unload(xspec,xibias)
   xspec[xibias]=nil
   collectgarbage()
end

function calc()
   title=species.." bias="..bias
   local defun=loadstring(inputs.func)
   if defun then 
      stool_bottom:deactivate_calc_button()
      defun(species)
      if bool(sview) then
       glv2DumpRequest(sview)
      end
      glv2UpdateSemiview(f)
      return true
   else 
      fl_show_alert("Syntax Error in Function Text.","Please correct.","")
      return false
   end
end



function on_reload_gli(self)
  print("lua on_reload_gli:",job_gli)
  
  reload_gli()
--  on_species_changed(self)
  
  objects.bias_browser=bias_browser(species)
  glv2SemiviewValChanged("objects.bias_browser")
  
end

function on_bias_changed(self)
  print("lua on_bias_changed:",species,ibias)
  
  calc()
end

function on_species_changed(self)
  print("lua on_species_changed:",species,"ibias=",ibias)
  
  objects.bias_browser=bias_browser(species)
  
  if ibias > nbias then
   print(" ibias=",ibias,"nbias=",nbias)
   ibias=1
   
   glv2SemiviewValChanged("objects.bias_browser")
   stool_bottom:set_inputs_func(species)
   
   glv2SemiviewValChanged("ibias")
   
  else
   glv2SemiviewValChanged("objects.bias_browser")
   stool_bottom:set_inputs_func(species)
   
   calc()
   glv2Update();
  end
  
end


animate=false
function on_animate(self)
  animate=not animate
  print("lua on_animate:",animate)
  
  if not animate then
    stool_right:activate_anibutton(false)
    glv2Update();
    print("animate stop")
    return
  end
  
  local oldibias=ibias
  
  print("animate start")
  
  stool_right:activate_browser(false)
  stool_bottom:deactivate()
  glv2Update();
  
  local dbias=stool_right:animate_step_size()
  
  for xibias=oldibias,nbias,dbias do
    ibias=xibias
    print("loading... ibias=",ibias)
    glv2SemiviewValChanged("ibias")
    glv2Update();
    print(" finished")
    if not animate then break end
  end
  
--[[
  print("animate stopped, restore old selection")
  ibias=oldibias
  glv2SemiviewValChanged("ibias")
  glv2Update();
]]
  
  stool_right:activate_anibutton(true)
  stool_right:activate_browser(true)
  stool_bottom:activate()
  glv2Update();
  
end




function on_calc(self)
  print("lua on_calc")
  calc()
end





forms={}
inputs={}
objects={}
callbacks={}
groups={}
visible={}


iline=0
objects.species_browser={}
for xspec,tab in xspecies do
   iline=iline+1
   objects.species_browser[iline]=xspec
end


ibias=1
bias=xspecies[species][ibias].bias
title=species.." bias="..bias

objects.bias_browser=bias_browser(species)


sview=glv2CreateSemiview(10,10,"pdelib2 semiview")
glv2StateFreezeAll(sview,true);

stool_right=glv2GetSemiviewRight()
stool_bottom=glv2GetSemiviewBottom()

swigext_extend(stool_right,"on_reload_gli")
stool_right.on_reload_gli=on_reload_gli

swigext_extend(stool_right,"on_bias_changed")
stool_right.on_bias_changed=on_bias_changed

swigext_extend(stool_right,"on_species_changed")
stool_right.on_species_changed=on_species_changed

swigext_extend(stool_right,"on_animate")
stool_right.on_animate=on_animate

swigext_extend(stool_bottom,"on_calc")
stool_bottom.on_calc=on_calc


stool_bottom:set_inputs_func(species)


calc()


glv2Run()
glv2CleanSemiview()




